package org.rcisoft.sys.sysuser.util;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.core.constant.CySysCons;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.sys.sysuser.dao.SysUserRepositorys;
import org.rcisoft.sys.sysuser.dto.ExportUserDTO;
import org.rcisoft.sys.sysuser.dto.UserErrorDTO;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.visit.dto.ExportVisitInfoDto;
import org.rcisoft.sys.visit.dto.ExportVisitUserDTO;
import org.rcisoft.sys.visit.dto.VisitInformationErrorDTO;
import org.rcisoft.sys.visit.dto.VisitPersonDTO;
import org.rcisoft.sys.wbac.user.dto.ErrorDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@ConditionalOnProperty(prefix = "cy.model", name = "permission", havingValue = CySysCons.WORK_BASE)
public class CheckUtil {


    /**
     * 对导入的用户信息进行非空校验
     *
     * @param sysUser 用户信息
     * @return
     */
    public static UserErrorDTO userNotNull(ExportUserDTO sysUser) {
        UserErrorDTO errorDTO = new UserErrorDTO();
        List<String> notTrue = new ArrayList<>();
        if (StringUtils.isEmpty(sysUser.getName()))
            notTrue.add("姓名不能为空");
        else if (sysUser.getName().length() > 32)
            notTrue.add("姓名超过最大长度32");
        if (StringUtils.isEmpty(sysUser.getPhone())) {
            notTrue.add("手机号不能为空");
        } else {
            if (!sysUser.getPhone().matches("^1[3-9]\\d{9}$"))
                notTrue.add("手机号格式不正确");
        }
        if (StringUtils.isEmpty(sysUser.getSex()))
            notTrue.add("性别不能为空");
        if (StringUtils.isEmpty(sysUser.getAddress()))
            notTrue.add("所属园区不能为空");
        if (StringUtils.isEmpty(sysUser.getDeptId()))
            notTrue.add("所属部门不能为空");
        if (StringUtils.isEmpty(sysUser.getPosition()))
            notTrue.add("职位不能为空");
        if (StringUtils.isEmpty(sysUser.getNltBz()))
            notTrue.add("班次不能为空");
        if (notTrue.size() > 0) {
            BeanUtil.copyProperties(sysUser, errorDTO);
            errorDTO.setError(notTrue.stream().collect(Collectors.joining(",")));
            return errorDTO;
        }
        return null;
    }

    /**
     * 对导入信息进行查重校验
     *
     * @param sysUser  导入用户信息
     * @param row      行数
     * @param sysUsers 已有用户信息
     * @return
     */
    public static ErrorDTO userRepeat(ExportUserDTO sysUser, int row, List<SysUser> sysUsers) {
        ErrorDTO errorDTO = new ErrorDTO();
        Map<Integer, List<String>> notTrues = new HashMap<>();
        List<String> notTrue = new ArrayList<>();
        errorDTO.setSuccess(0);
        for (SysUser existenceUser : sysUsers) {
            if (existenceUser.getUsername().equals(sysUser.getUsername())) {
                notTrue.add("登录名不能重复");
            }
            if (existenceUser.getPhone().equals(sysUser.getPhone())) {
                notTrue.add("手机号不能重复");
            }
        }
        if (notTrue.size() > 0) {
            errorDTO.setSuccess(1);
            notTrues.put(row, notTrue);
            errorDTO.setMap(notTrues);
        }
        return errorDTO;
    }

    public static VisitInformationErrorDTO infoNotNull(ExportVisitInfoDto visitInfoDto) {
        VisitInformationErrorDTO errorDTO = new VisitInformationErrorDTO();
        List<String> notTrue = new ArrayList<>();
        if (StringUtils.isEmpty(visitInfoDto.getUserName()))
            notTrue.add("邀请人姓名不能为空");
        if (StringUtils.isEmpty(visitInfoDto.getPhone()))
            notTrue.add("邀请人手机号不能为空");
        if (StringUtils.isEmpty(visitInfoDto.getVisitReason()))
            notTrue.add("访问事由不能为空");
        if (StringUtils.isEmpty(visitInfoDto.getVisitRegion()))
            notTrue.add("访问区域不能为空");
        if (!isValid(visitInfoDto.getVisitDate(), DateTimeFormatter.ofPattern("yyyy-MM-dd", Locale.CHINA))) {
            notTrue.add("访问日期为空或不符合规范");
        }
        if (!isValid(visitInfoDto.getVisitTime(), DateTimeFormatter.ofPattern("HH:mm", Locale.CHINA))) {
            notTrue.add("访问时间为空或不符合规范");
        }
        if (StringUtils.isEmpty(visitInfoDto.getVisitName()))
            notTrue.add("访客姓名不能为空");
        if (StringUtils.isEmpty(visitInfoDto.getVisitPhone()))
            notTrue.add("访客手机号不能为空");
        // 判断访客姓名和手机号是否数量匹配
        if (StringUtils.isNotEmpty(visitInfoDto.getVisitName()) && StringUtils.isNotEmpty(visitInfoDto.getVisitPhone())) {
            List<String> userName = Arrays.asList(visitInfoDto.getVisitName().split(",|，")).stream()
                    .filter(u -> StringUtils.isNotEmpty(u)).collect(Collectors.toList());
            List<String> phone = Arrays.asList(visitInfoDto.getVisitPhone().split(",|，")).stream()
                    .filter(p -> StringUtils.isNotEmpty(p)).distinct()
                    .collect(Collectors.toList());
            if (userName.size() != phone.size())
                notTrue.add("访客姓名和访客手机号数量不对应");
            List<String> matchList = phone.stream().filter(p -> p.matches("^1[3-9]\\d{9}$")).collect(Collectors.toList());
            if (matchList.size() < phone.size())
                notTrue.add("访客手机号不符合规范");
            for (String temp:userName){
                if (temp.length() > 32){
                    notTrue.add("访客姓名超出32个字符");
                    continue;
                }
            }
            for (String temp:phone){
                if (temp.length() != 11){
                    notTrue.add("访客手机号格式错误");
                    continue;
                }
                String regex = "^((13[0-9])|(14[5|7])|(15([0-3]|[5-9]))|(17[013678])|(18[0,5-9]))\\d{8}$";
                Pattern p = Pattern.compile(regex);
                Matcher m = p.matcher(temp);
                boolean isMatch = m.matches();
                if (!isMatch){
                    notTrue.add("访客手机号格式错误");
                    continue;
                }
            }
        }
        if (notTrue.size() > 0) {
            BeanUtil.copyProperties(visitInfoDto, errorDTO);
            errorDTO.setError(notTrue.stream().collect(Collectors.joining(",")));
            return errorDTO;
        }
        return null;
    }

    public static boolean isValid(String dateStr, DateTimeFormatter dateFormatter) {
        if (StringUtils.isEmpty(dateStr))
            return false;
        try {
            dateFormatter.parse(dateStr);
        } catch (DateTimeParseException e) {
            return false;
        }
        return true;
    }

    public static VisitPersonDTO visvitNotNull(ExportVisitUserDTO sysUser) {

        VisitPersonDTO errorDTO = new VisitPersonDTO();
        Map<Integer, List<String>> notTrues = new HashMap<>();
        List<String> notTrue = new ArrayList<>();
//        errorDTO.setSuccess(0);
        if (StringUtils.isEmpty(sysUser.getName())) {
            notTrue.add("姓名不能为空");
        }
        if (StringUtils.isEmpty(sysUser.getPhone())) {
            notTrue.add("手机号不能为空");
        }
        if (!StringUtils.isEmpty(sysUser.getPhone())) {

            List<String> phone = Arrays.asList(sysUser.getPhone().split(",|，")).stream()
                    .filter(p -> StringUtils.isNotEmpty(p)).distinct().collect(Collectors.toList());
            List<String> matchList = phone.stream().filter(p -> p.matches("^1[3-9]\\d{9}$")).collect(Collectors.toList());
            if (matchList.size() < phone.size())
                notTrue.add("访客手机号不符合规范");
        }


        if (notTrue.size() > 0) {
            BeanUtil.copyProperties(sysUser, errorDTO);
            errorDTO.setError(notTrue.stream().collect(Collectors.joining(",")));
//            errorDTO.setSuccess(1);

//            notTrues.put(index, notTrue);
//            errorDTO.setMap(notTrues);
            return errorDTO;
        }
        return null;
    }
}
